@@ -1,5 +1,5 @@
 """
-SimpleEval - (C) 2013-2018 Daniel Fairhead
+SimpleEval - (C) 2013-2019 Daniel Fairhead
 -------------------------------------
 
 An short, easy to use, safe and reasonably extensible expression evaluator.
@@ -46,10 +46,11 @@ Contributors:
 - xaled (Khalid Grandi) method chaining correctly, double-eval bugfix.
 - EdwardBetts (Edward Betts) spelling correction.
 - charlax (Charles-Axel Dein charlax) Makefile and cleanups
-- mommothazaz123 (Andrew Zhu) f"string" support
+- mommothazaz123 (Andrew Zhu) f"string" support, Python 3.8 support
 - lubieowoce (Uryga) various potential vulnerabilities
 - JCavallo (Jean Cavallo) names dict shouldn't be modified
-
+- Birne94 (Daniel Birnstiel) for fixing leaking generators.
+- patricksurry (Patrick Surry) or should return last value, even if falsy.
 
 -------------------------------------
 Basic Usage:
@@ -94,6 +95,8 @@ import operator as op
 import sys
 from random import random
 
+PYTHON3 = sys.version_info[0] == 3
+
 ########################################
 # Module wide 'globals'
 
@@ -101,9 +104,19 @@ MAX_STRING_LENGTH = 100000
 MAX_COMPREHENSION_LENGTH = 10000
 MAX_POWER = 4000000  # highest exponent
 DISALLOW_PREFIXES = ['_', 'func_']
-DISALLOW_METHODS = ['format', 'mro']
+DISALLOW_METHODS = ['format', 'format_map', 'mro']
 
-PYTHON3 = sys.version_info[0] == 3
+# Disallow functions:
+# This, strictly speaking, is not necessary.  These /should/ never be accessable anyway,
+# if DISALLOW_PREFIXES and DISALLOW_METHODS are all right.  This is here to try and help
+# people not be stupid.  Allowing these functions opens up all sorts of holes - if any of
+# their functionality is required, then please wrap them up in a safe container.  And think
+# very hard about it first.  And don't say I didn't warn you.
+
+DISALLOW_FUNCTIONS = {type, isinstance, eval, getattr, setattr, help, repr, compile, open}
+
+if PYTHON3:
+    exec('DISALLOW_FUNCTIONS.add(exec)') # exec is not a function in Python2...
 
 
 ########################################
@@ -256,11 +269,11 @@ class SimpleEval(object):  # pylint: disable=too-few-public-methods
             functions (add, random, get_val, whatever) and names. """
 
         if not operators:
-            operators = DEFAULT_OPERATORS
+            operators = DEFAULT_OPERATORS.copy()
         if not functions:
-            functions = DEFAULT_FUNCTIONS
+            functions = DEFAULT_FUNCTIONS.copy()
         if not names:
-            names = DEFAULT_NAMES
+            names = DEFAULT_NAMES.copy()
 
         self.operators = operators
         self.functions = functions
@@ -285,16 +298,26 @@ class SimpleEval(object):  # pylint: disable=too-few-public-methods
 
         # py3k stuff:
         if hasattr(ast, 'NameConstant'):
-            self.nodes[ast.NameConstant] = self._eval_nameconstant
+            self.nodes[ast.NameConstant] = self._eval_constant
 
         # py3.6, f-strings
         if hasattr(ast, 'JoinedStr'):
             self.nodes[ast.JoinedStr] = self._eval_joinedstr  # f-string
             self.nodes[ast.FormattedValue] = self._eval_formattedvalue  # formatted value in f-string
 
+        # py3.8 uses ast.Constant instead of ast.Num, ast.Str, ast.NameConstant
+        if hasattr(ast, 'Constant'):
+            self.nodes[ast.Constant] = self._eval_constant
+
         # Defaults:
 
         self.ATTR_INDEX_FALLBACK = ATTR_INDEX_FALLBACK
+        
+        # Check for forbidden functions:
+
+        for f in list(self.functions.values()):
+            if f in DISALLOW_FUNCTIONS:
+                raise FeatureNotAvailable('This function {} is a really bad idea.'.format(f))
 
 
     def eval(self, expr):
@@ -332,7 +355,10 @@ class SimpleEval(object):  # pylint: disable=too-few-public-methods
         return node.s
 
     @staticmethod
-    def _eval_nameconstant(node):
+    def _eval_constant(node):
+        if hasattr(node.value, '__len__') and len(node.value) > MAX_STRING_LENGTH:
+            raise IterableTooLong("Literal in statement is too long!"
+                                  " ({0}, when {1} is max)".format(len(node.value), MAX_STRING_LENGTH))
         return node.value
 
     def _eval_unaryop(self, node):
@@ -348,14 +374,14 @@ class SimpleEval(object):  # pylint: disable=too-few-public-methods
             for value in node.values:
                 vout = self._eval(value)
                 if not vout:
-                    return False
+                    return vout
             return vout
         elif isinstance(node.op, ast.Or):
             for value in node.values:
                 vout = self._eval(value)
                 if vout:
                     return vout
-            return False
+            return vout
 
     def _eval_compare(self, node):
         right = self._eval(node.left)
@@ -383,6 +409,9 @@ class SimpleEval(object):  # pylint: disable=too-few-public-methods
             except AttributeError as e:
                 raise FeatureNotAvailable('Lambda Functions not implemented')
 
+            if func in DISALLOW_FUNCTIONS:
+                raise FeatureNotAvailable('This function is forbidden')
+
         return func(
             *(self._eval(a) for a in node.args),
             **dict(self._eval(k) for k in node.keywords)
@@ -564,9 +593,10 @@ class EvalWithCompoundTypes(SimpleEval):
                     else:
                         to_return.append(self._eval(node.elt))
 
-        do_generator()
-
-        self.nodes.update({ast.Name: previous_name_evaller})
+        try:
+            do_generator()
+        finally:
+            self.nodes.update({ast.Name: previous_name_evaller})
 
         return to_return
 
